package de.hdm_stuttgart.battlearena.Persistance.Classes;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;


import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Properties;
import java.sql.*;

public class AzureDB implements IDataBase {

    private static final Logger log = LogManager.getLogger(AzureDB.class);


    @Override
    public Connection connect() throws IOException, SQLException {

        log.info("Loading the properties File!");
        Properties properties = new Properties();
        properties.load(getClass().getClassLoader().getResourceAsStream("config.properties"));

        log.info("Connecting to the database!");
        Connection connection = DriverManager.getConnection(properties.getProperty("url"), properties.getProperty("user"), properties.getProperty("password"));
        log.info("Database connection test" + connection.getCatalog());

        connection.setAutoCommit(true);

        return connection;
    }


    @Override
    public void createNewPlayer(String playerName) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String sql = "INSERT INTO players (player_name, games_won, games_lost, kills, deaths, blocks_destroyed, ingame_time)" + "VALUES (?,?,?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, playerName);
            preparedStatement.setInt(2, 0);
            preparedStatement.setInt(3, 0);
            preparedStatement.setInt(4, 0);
            preparedStatement.setInt(5, 0);
            preparedStatement.setInt(6, 0);
            preparedStatement.setInt(7, 0);

            log.info("Inserting new player");
            preparedStatement.execute();
        }
    }

    @Override
    public void createMap(String name, int length, int width, String mapData) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String mapSize = length + "X" + width;

            String sql = "INSERT INTO battleArena.dbo.maps (map_name, map_size, map_data, map_version, map_hash) VALUES(?,?,?,?,?)";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, name);
            preparedStatement.setString(2, mapSize);
            preparedStatement.setString(3, mapData);

            log.info("Inserting new map");
            preparedStatement.execute();
        }
    }

    @Override
    public LinkedHashMap<String, String> getMapNames() throws SQLException, IOException {
        LinkedHashMap<String, String> mapNames = new LinkedHashMap<>();

        try(Connection connection = connect()) {
            String sql = "SELECT maps.map_id, maps.map_name FROM maps";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            ResultSet results = preparedStatement.executeQuery();
            log.info("Getting map names");
            while (results.next()) {
                mapNames.put(results.getString("map_id"), results.getString("map_name"));
            }
            connection.close();
            return mapNames;
        }
    }

    @Override
    public String getMapByID(String ID) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String sql = "SELECT map_name FROM maps WHERE map_id = ?";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, ID);
            log.info("Getting Map...");
            String map = preparedStatement.executeQuery().toString();
            return map;
        }
    }

    @Override
    public String getMapSizeByID(String ID) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String sql = "SELECT map_size FROM maps WHERE battleArena.dbo.maps.map_id=?";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, ID);
            ResultSet result = preparedStatement.executeQuery();
            log.info("getting map size...");
            String mapSize = result.getString("map_size");
            return mapSize;
        }
    }

    @Override
    public ResultSet getStatistics(String playerName) throws SQLException, IOException {
        try(Connection connection = connect()){
        String sql = "SELECT * FROM players where player_name = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setString(1, playerName);
        ResultSet resultSet = preparedStatement.executeQuery();
        log.info("getting statistics...");
        return resultSet;
        }
    }

    @Override
    public void updateMap(String mapID, String mapName, String mapData) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String sql = "UPDATE maps SET map_name = ?, map_data = ? WHERE map_id = ?";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, mapName);
            preparedStatement.setString(2, mapData);
            preparedStatement.setString(3, mapID);
            preparedStatement.execute();
            log.info("updating map in Database");
        }
    }


    @Override
    public void updatePlayerName(String playerNameNew, String playerNameOld) throws SQLException, IOException {
        try(Connection connection = connect()){
        String sql = "UPDATE players SET player_name = ? WHERE player_name = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setString(1, playerNameNew);
        preparedStatement.setString(2, playerNameOld);
        preparedStatement.execute();
        log.info("updating player name in Database...");
        }
    }

    @Override
    public void updatePlayerStatistics(String playerName, int gamesWon, int gamesLost, int kills, int deaths, int blocksDestroyed, int gameTime) throws SQLException, IOException {
        try (Connection connection = connect()){
        String sql = "UPDATE players SET games_won = ?, games_lost = ?, kills = ?, " + "deaths = ?, blocks_destroyed = ?, ingame_time = ? WHERE player_name = ?";
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setInt(1, gamesWon);
        preparedStatement.setInt(2, gamesLost);
        preparedStatement.setInt(3, kills);
        preparedStatement.setInt(4, deaths);
        preparedStatement.setInt(5, blocksDestroyed);
        preparedStatement.setInt(6, gameTime);
        preparedStatement.setString(7, playerName);
        preparedStatement.execute();
        log.info("updating player statistics");

        }
    }


    @Override
    public void deletePlayer(String playerName) throws IOException, SQLException {
        try(Connection connection = connect()) {
            String sql = "DELETE FROM players WHERE player_name=?";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, playerName);
            preparedStatement.execute();
            log.info("deleting player");

        }
    }

    @Override
    public void deleteMap(String mapID) throws SQLException, IOException {
        try(Connection connection = connect()) {
            String sql = "DELETE * FROM maps WHERE map_id = ?";
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            preparedStatement.setString(1, mapID);
            preparedStatement.execute();
            log.info("deleting map...");
        }
    }


}
